package main

import (
	// Standard
	"flag"
	"fmt"
	"html/template"
	"log"
	"net/http"
	"os"
	"path"
	"regexp"
	"strings"
)

const VERSION = "v0.0.1";
const LISTEN_HOST = "0.0.0.0";

var (
	flagPort int
	connectionManager ConnectionManager
	static http.Handler
	config Config
)


func init() {// {{{
	flag.IntVar(
		&flagPort,
		"port",
		1371,
		"TCP port to listen on",
	)

	flag.Parse()
}// }}}
func main() {// {{{
	var err error
	log.Printf("\x1b[32mNotes\x1b[0m %s\n", VERSION)

	config, err = ConfigRead(os.Getenv("HOME")+"/.config/notes.yaml")
	if err != nil {
		fmt.Printf("%s\n", err)
		os.Exit(1)
	}

	dbInit()
	connectionManager = NewConnectionManager()
	go connectionManager.BroadcastLoop()

	static = http.FileServer(http.Dir("./static"))
	http.HandleFunc("/css_updated", cssUpdateHandler)
	http.HandleFunc("/session/create", sessionCreate)
	http.HandleFunc("/session/retrieve", sessionRetrieve)
	http.HandleFunc("/session/authenticate", sessionAuthenticate)
	http.HandleFunc("/ws", websocketHandler)
	http.HandleFunc("/", staticHandler)

	listen := fmt.Sprintf("%s:%d", LISTEN_HOST, flagPort)
	log.Printf("\x1b[32mNotes\x1b[0m Listening on %s\n", listen)
	log.Printf("\x1b[32mNotes\x1b[0m Answer for domains %s\n", strings.Join(config.Websocket.Domains, ", "))
	http.ListenAndServe(listen, nil)
}// }}}

func cssUpdateHandler(w http.ResponseWriter, r *http.Request) {// {{{
	log.Println("[BROADCAST] CSS updated")
	connectionManager.Broadcast(struct{ Ok bool; ID string; Op string }{ Ok: true, Op: "css_reload" })
}// }}}
func sessionCreate(w http.ResponseWriter, r *http.Request) {// {{{
	session, err := NewSession()
	if err != nil {
		responseError(w, err)
		return
	}
	responseData(w, map[string]interface{}{
		"OK": true,
		"Session": session,
	})
}// }}}
func sessionRetrieve(w http.ResponseWriter, r *http.Request) {// {{{
	var err error
	var uuid string

	if uuid, err = sessionUUID(r); err != nil {
		responseError(w, err)
		return
	}

	session := Session{ UUID: uuid }
	if err = session.Retrieve(); err != nil {
		responseError(w, err)
		return
	}

	responseData(w, map[string]interface{}{
		"OK": true,
		"Session": session,
	})
}// }}}
func sessionAuthenticate(w http.ResponseWriter, r *http.Request) {// {{{
	var err error
	var uuid string

	if uuid, err = sessionUUID(r); err != nil {
		responseError(w, err)
		return
	}

	session := Session{
		UUID: uuid,
	}

	req := struct{ Username string; Password string }{}
	if err = request(r, &req); err != nil {
		responseError(w, err)
		return
	}

	if err = session.Authenticate(req.Username, req.Password); err != nil {
		responseError(w, err)
		return
	}

	responseData(w, map[string]interface{}{
		"OK": true,
		"Session": session,
	})
}// }}}
func websocketHandler(w http.ResponseWriter, r *http.Request) {// {{{
	var err error

	_, err = connectionManager.NewConnection(w, r)
	if err != nil {
		log.Printf("[Connection] %s\n", err)
		return
	}
}// }}}
func staticHandler(w http.ResponseWriter, r *http.Request) {// {{{
	data := struct{
		VERSION string
	}{
		VERSION: VERSION,
	}

	// URLs with pattern /(css|images)/v1.0.0/foobar are stripped of the version.
	// To get rid of problems with cached content in browser on a new version release,
	// while also not disabling cache altogether.
	rxp := regexp.MustCompile("^/(css|images|js|fonts)/v[0-9]+\\.[0-9]+\\.[0-9]+/(.*)$")
	if comp := rxp.FindStringSubmatch(r.URL.Path); comp != nil {
		r.URL.Path = fmt.Sprintf("/%s/%s", comp[1], comp[2])
		static.ServeHTTP(w, r)
		return
	}

	// Everything else is run through the template system.
	// For now to get VERSION into files to fix caching.
	tmpl, err := newTemplate(r.URL.Path)
	if err != nil {
		if os.IsNotExist(err) {
			w.WriteHeader(404)
		}
		w.Write([]byte(err.Error()))
		return
	}

	if err = tmpl.Execute(w, data); err != nil {
		w.Write([]byte(err.Error()))
	}
}// }}}

func newTemplate(requestPath string) (tmpl *template.Template, err error) {// {{{
	// Append index.html if needed for further reading of the file
	p := requestPath
	if p[len(p)-1] == '/' {
		p += "index.html"
	}
	p = "static" + p

	base := path.Base(p)
	if tmpl, err = template.New(base).ParseFiles(p); err != nil { return }

	return
}// }}}

// vim: foldmethod=marker
